class BuildInfo:  # pylint: disable=too-few-public-methods
    python_version = '3.10.15'
    os_name = 'posix:ubuntu'
    version = '1.0.8'
    git_sha = '80fa8dcd6c2f7acc77c515f2b74affa646b26e03'
    git_branch = 'master'
    git_uncommitted = [
        'M  iuv/test_iuv_logger.py',
        'M  iuv/test_iuv_report.py',
        'D  medver_pytest/__init__.py',
        'D  medver_pytest/lib/.gitignore',
        'R  medver_pytest/lib/__init__.py -> src/__init__.py',
        'A  src/medver_pytest/.gitignore',
        'A  src/medver_pytest/__init__.py',
        'AM src/medver_pytest/build_info.py',
        'R  medver_pytest/lib/cfg.py -> src/medver_pytest/cfg.py',
        'R  medver_pytest/lib/cli.py -> src/medver_pytest/cli.py',
        'R  medver_pytest/lib/constants.py -> src/medver_pytest/constants.py',
        'RM medver_pytest/lib/constants_version.py -> src/medver_pytest/constants_version.py',
        'R  medver_pytest/lib/harness.py -> src/medver_pytest/harness.py',
        'R  medver_pytest/lib/iuv/__init__.py -> src/medver_pytest/iuv/__init__.py',
        'R  medver_pytest/lib/iuv/iuv.py -> src/medver_pytest/iuv/iuv.py',
        'R  medver_pytest/lib/iuv/iuv_data.py -> src/medver_pytest/iuv/iuv_data.py',
        'R  medver_pytest/lib/iuv/iuv_requirements.py -> src/medver_pytest/iuv/iuv_requirements.py',
        'R  medver_pytest/lib/report/__init__.py -> src/medver_pytest/log/__init__.py',
        'R  medver_pytest/lib/log/logger.py -> src/medver_pytest/log/logger.py',
        'R  medver_pytest/lib/log/logger_stdout.py -> src/medver_pytest/log/logger_stdout.py',
        'R  medver_pytest/lib/os_specific.py -> src/medver_pytest/os_specific.py',
        'R  medver_pytest/lib/protocol.py -> src/medver_pytest/protocol.py',
        'R  medver_pytest/lib/report/common/__init__.py -> src/medver_pytest/report/__init__.py',
        'R  medver_pytest/lib/report/protocol/__init__.py -> src/medver_pytest/report/common/__init__.py',
        'R  medver_pytest/lib/report/common/footer.py -> src/medver_pytest/report/common/footer.py',
        'R  medver_pytest/lib/report/common/gen_base_docx.py -> src/medver_pytest/report/common/gen_base_docx.py',
        'R  medver_pytest/lib/report/common/gen_base_minhtml.py -> src/medver_pytest/report/common/gen_base_minhtml.py',
        'R  medver_pytest/lib/report/common/gen_base_pdf.py -> src/medver_pytest/report/common/gen_base_pdf.py',
        'R  medver_pytest/lib/report/common/gen_base_txt.py -> src/medver_pytest/report/common/gen_base_txt.py',
        'R  medver_pytest/lib/report/common/header.py -> src/medver_pytest/report/common/header.py',
        'R  medver_pytest/lib/report/common/page_details.py -> src/medver_pytest/report/common/page_details.py',
        'R  medver_pytest/lib/report/common/page_info.py -> src/medver_pytest/report/common/page_info.py',
        'R  medver_pytest/lib/report/summary/__init__.py -> src/medver_pytest/report/protocol/__init__.py',
        'R  medver_pytest/lib/report/protocol/gen_tp_base.py -> src/medver_pytest/report/protocol/gen_tp_base.py',
        'R  medver_pytest/lib/report/protocol/gen_tp_docx.py -> src/medver_pytest/report/protocol/gen_tp_docx.py',
        'R  medver_pytest/lib/report/protocol/gen_tp_minhtml.py -> src/medver_pytest/report/protocol/gen_tp_minhtml.py',
        'R  medver_pytest/lib/report/protocol/gen_tp_pdf.py -> src/medver_pytest/report/protocol/gen_tp_pdf.py',
        'R  medver_pytest/lib/report/protocol/gen_tp_txt.py -> src/medver_pytest/report/protocol/gen_tp_txt.py',
        'R  medver_pytest/lib/report/report.py -> src/medver_pytest/report/report.py',
        'R  medver_pytest/lib/report/trace_matrix/__init__.py -> src/medver_pytest/report/summary/__init__.py',
        'R  medver_pytest/lib/report/summary/gen_summary_base.py -> src/medver_pytest/report/summary/gen_summary_base.py',
        'R  medver_pytest/lib/report/summary/gen_summary_docx.py -> src/medver_pytest/report/summary/gen_summary_docx.py',
        'R  medver_pytest/lib/report/summary/gen_summary_minhtml.py -> src/medver_pytest/report/summary/gen_summary_minhtml.py',
        'R  medver_pytest/lib/report/summary/gen_summary_pdf.py -> src/medver_pytest/report/summary/gen_summary_pdf.py',
        'R  medver_pytest/lib/report/summary/gen_summary_txt.py -> src/medver_pytest/report/summary/gen_summary_txt.py',
        'R  medver_pytest/lib/view/__init__.py -> src/medver_pytest/report/trace_matrix/__init__.py',
        'R  medver_pytest/lib/report/trace_matrix/gen_trace_matrix_base.py -> src/medver_pytest/report/trace_matrix/gen_trace_matrix_base.py',
        'R  medver_pytest/lib/report/trace_matrix/gen_trace_matrix_docx.py -> src/medver_pytest/report/trace_matrix/gen_trace_matrix_docx.py',
        'R  medver_pytest/lib/report/trace_matrix/gen_trace_matrix_minhtml.py -> src/medver_pytest/report/trace_matrix/gen_trace_matrix_minhtml.py',
        'R  medver_pytest/lib/report/trace_matrix/gen_trace_matrix_pdf.py -> src/medver_pytest/report/trace_matrix/gen_trace_matrix_pdf.py',
        'R  medver_pytest/lib/report/trace_matrix/gen_trace_matrix_txt.py -> src/medver_pytest/report/trace_matrix/gen_trace_matrix_txt.py',
        'R  medver_pytest/lib/requirements.py -> src/medver_pytest/requirements.py',
        'R  medver_pytest/lib/result_formatter.py -> src/medver_pytest/result_formatter.py',
        'R  medver_pytest/lib/result_summary.py -> src/medver_pytest/result_summary.py',
        'R  medver_pytest/lib/services.py -> src/medver_pytest/services.py',
        'R  medver_pytest/lib/status_info.py -> src/medver_pytest/status_info.py',
        'R  medver_pytest/lib/stdout_logger.py -> src/medver_pytest/stdout_logger.py',
        'A  src/medver_pytest/storage/__init__.py',
        'R  medver_pytest/lib/storage/storage.py -> src/medver_pytest/storage/storage.py',
        'R  medver_pytest/lib/storage/storage_dev.py -> src/medver_pytest/storage/storage_dev.py',
        'R  medver_pytest/lib/storage/storage_file_base.py -> src/medver_pytest/storage/storage_file_base.py',
        'R  medver_pytest/lib/storage/storage_shared_drive.py -> src/medver_pytest/storage/storage_shared_drive.py',
        'R  medver_pytest/lib/summary.py -> src/medver_pytest/summary.py',
        'R  medver_pytest/lib/trace_matrix.py -> src/medver_pytest/trace_matrix.py',
        'R  medver_pytest/lib/utils.py -> src/medver_pytest/utils.py',
        'R  medver_pytest/lib/verifier.py -> src/medver_pytest/verifier.py',
        'A  src/medver_pytest/view/__init__.py',
        'R  medver_pytest/lib/view/dlg_base.py -> src/medver_pytest/view/dlg_base.py',
        'R  medver_pytest/lib/view/dlg_manual_action.py -> src/medver_pytest/view/dlg_manual_action.py',
        'R  medver_pytest/lib/view/dlg_manual_verify.py -> src/medver_pytest/view/dlg_manual_verify.py',
        'R  medver_pytest/lib/view/view.py -> src/medver_pytest/view/view.py',
        'MM tools/pyproject_template.toml',
        ' m tools/xplat_utils',
        'M  ut/mocks/mock_protocol.py',
        'M  ut/test_ut_cfg.py',
        'M  ut/test_ut_cli.py',
        'M  ut/test_ut_harness.py',
        'M  ut/test_ut_page_details.py',
        'M  ut/test_ut_requirements.py',
        'M  ut/test_ut_result_summary.py',
        'M  ut/test_ut_storage.py',
        'M  ut/test_ut_utils.py',
        'M  ut/test_ut_verify_delta.py',
        'M  ut/test_ut_verify_delta_pct.py',
        'M  xplat.cfg',
    ]
    git_unpushed = [
    ]
